from django.conf import settings
from django.utils.translation import ugettext_lazy as _


ASSIGNMENT_ANY = 0
ASSIGNMENT_MATCH = 1
ASSIGNMENT_EXCEPT = 2
ASSIGNMENT_CHOICES = (
    (ASSIGNMENT_ANY, _('any')),
    (ASSIGNMENT_MATCH, _("matches")),
    (ASSIGNMENT_EXCEPT, _("don't match")),
)

DJANGO_ADMIN_SSO_ADD_LOGIN_BUTTON = getattr(
    settings,
    'DJANGO_ADMIN_SSO_ADD_LOGIN_BUTTON',
    True)

AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')

DJANGO_ADMIN_SSO_OAUTH_CLIENT_ID = getattr(
    settings,
    'DJANGO_ADMIN_SSO_OAUTH_CLIENT_ID',
    None)
DJANGO_ADMIN_SSO_OAUTH_CLIENT_SECRET = getattr(
    settings,
    'DJANGO_ADMIN_SSO_OAUTH_CLIENT_SECRET',
    None)

DJANGO_ADMIN_SSO_AUTH_URI = getattr(
    settings,
    'DJANGO_ADMIN_SSO_AUTH_URI',
    'https://accounts.google.com/o/oauth2/auth')
DJANGO_ADMIN_SSO_TOKEN_URI = getattr(
    settings,
    'DJANGO_ADMIN_SSO_TOKEN_URI',
    'https://accounts.google.com/o/oauth2/token')
DJANGO_ADMIN_SSO_REVOKE_URI = getattr(
    settings,
    'DJANGO_ADMIN_SSO_REVOKE_URI',
    'https://accounts.google.com/o/oauth2/revoke')



from django.conf.urls import url
from django.contrib import admin

from admin_sso import settings
from admin_sso.models import Assignment


class AssignmentAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'username', 'username_mode', 'domain',
                    'user', 'weight')
    list_editable = ('username', 'username_mode', 'domain', 'user', 'weight')

    def get_urls(self):
        from admin_sso.views import start, end

        info = (self.model._meta.app_label, self.model._meta.model_name)
        return [
            url(r'^start/$', start,
                name='%s_%s_start' % info),
            url(r'^end/$', end,
                name='%s_%s_end' % info),
        ] + super(AssignmentAdmin, self).get_urls()

admin.site.register(Assignment, AssignmentAdmin)


if settings.DJANGO_ADMIN_SSO_ADD_LOGIN_BUTTON:
    admin.site.login_template = 'admin_sso/login.html'



VERSION = (2, 1, 0)
__version__ = '.'.join(map(str, VERSION))

default_app_config = 'admin_sso.apps.AdminSSOConfig'

# Do not use Django settings at module level as recommended
try:
    from django.utils.functional import LazyObject
except ImportError:
    pass
else:
    class LazySettings(LazyObject):
        def _setup(self):
            from admin_sso import default_settings
            self._wrapped = Settings(default_settings)

    class Settings(object):
        def __init__(self, settings_module):
            for setting in dir(settings_module):
                if setting == setting.upper():
                    setattr(self, setting, getattr(settings_module, setting))

    settings = LazySettings()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Assignment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('username_mode', models.IntegerField(choices=[(0, 'any'), (1, 'matches'), (2, "don't match")])),
                ('username', models.CharField(max_length=255, blank=True)),
                ('domain', models.CharField(max_length=255)),
                ('copy', models.BooleanField(default=False)),
                ('weight', models.PositiveIntegerField(default=0)),
                ('user', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ('-weight',),
                'verbose_name': 'Assignment',
                'verbose_name_plural': 'Assignments',
            },
        ),
    ]



import fnmatch
from django.db import models
from django.utils.translation import ugettext_lazy as _

from admin_sso import settings


class AssignmentManager(models.Manager):
    def for_email(self, email):
        if not email:
            return None

        try:
            username, domain = email.split('@')
        except ValueError:
            return None
        possible_assignments = self.filter(domain=domain)
        used_assignment = None
        for assignment in possible_assignments:
            if assignment.username_mode == settings.ASSIGNMENT_ANY:
                used_assignment = assignment
                break
            elif assignment.username_mode == settings.ASSIGNMENT_MATCH:
                if fnmatch.fnmatch(username, assignment.username):
                    used_assignment = assignment
                    break
            elif assignment.username_mode == settings.ASSIGNMENT_EXCEPT:
                if not fnmatch.fnmatch(username, assignment.username):
                    used_assignment = assignment
                    break
        if used_assignment is None:
            return None
        return used_assignment


class Assignment(models.Model):
    username_mode = models.IntegerField(choices=settings.ASSIGNMENT_CHOICES)
    username = models.CharField(max_length=255, blank=True)
    domain = models.CharField(max_length=255)
    copy = models.BooleanField(default=False)
    weight = models.PositiveIntegerField(default=0)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True,
                             on_delete=models.CASCADE)

    class Meta:
        verbose_name = _('Assignment')
        verbose_name_plural = _('Assignments')
        ordering = ('-weight',)

    def __unicode__(self):
        return u"%s(%s) @%s" % (dict(settings.ASSIGNMENT_CHOICES)[self.username_mode], self.username, self.domain)

    objects = AssignmentManager()



#! /usr/bin/env python
import os
from setuptools import setup

import admin_sso
setup(
    name='django-admin-sso',
    version=admin_sso.__version__,
    description='Django SSO solution',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author='Marc Egli',
    author_email='egli@allink.ch',
    url='http://github.com/matthiask/django-admin-sso/',
    license='BSD License',
    platforms=['OS Independent'],
    packages=[
        'admin_sso',
        'admin_sso.migrations',
    ],
    # package_data={'admin_sso':'templates/*.html'},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=(
        'Django>=1.7',
        'oauth2client>=1.2',
    ),
    include_package_data=True,
)



from django.core.urlresolvers import reverse
from django.contrib.auth import authenticate, login
from django.http import HttpResponseRedirect

from oauth2client.client import OAuth2WebServerFlow, FlowExchangeError

from admin_sso import settings

flow_kwargs = {
    'client_id': settings.DJANGO_ADMIN_SSO_OAUTH_CLIENT_ID,
    'client_secret': settings.DJANGO_ADMIN_SSO_OAUTH_CLIENT_SECRET,
    'scope': 'email',
}
if settings.DJANGO_ADMIN_SSO_AUTH_URI:
    flow_kwargs['auth_uri'] = settings.DJANGO_ADMIN_SSO_AUTH_URI

if settings.DJANGO_ADMIN_SSO_TOKEN_URI:
    flow_kwargs['token_uri'] = settings.DJANGO_ADMIN_SSO_TOKEN_URI

if settings.DJANGO_ADMIN_SSO_REVOKE_URI:
    flow_kwargs['revoke_uri'] = settings.DJANGO_ADMIN_SSO_REVOKE_URI

flow_override = None


def start(request):
    flow = OAuth2WebServerFlow(
        redirect_uri=request.build_absolute_uri(
            reverse('admin:admin_sso_assignment_end')),
        **flow_kwargs)

    return HttpResponseRedirect(flow.step1_get_authorize_url())


def end(request):
    if flow_override is None:
        flow = OAuth2WebServerFlow(
            redirect_uri=request.build_absolute_uri(
                reverse('admin:admin_sso_assignment_end')),
            **flow_kwargs)
    else:
        flow = flow_override

    code = request.GET.get('code', None)
    if not code:
        return HttpResponseRedirect(reverse('admin:index'))
    try:
        credentials = flow.step2_exchange(code)
    except FlowExchangeError:
        return HttpResponseRedirect(reverse('admin:index'))

    if credentials.id_token['email_verified']:
        email = credentials.id_token['email']
        user = authenticate(sso_email=email)
        if user and user.is_active:
            login(request, user)
            return HttpResponseRedirect(reverse('admin:index'))

    # if anything fails redirect to admin:index
    return HttpResponseRedirect(reverse('admin:index'))
